@@ -46,7 +46,19 @@ use std::sync::Arc;
 use parking_lot::Mutex;
 use wasmer::{imports, Function, ImportObject, Module};
 
-use crate::{host_environment::HostEnvironment, imported_fn::comments::get_leading_comments_proxy};
+use crate::{
+    host_environment::BaseHostEnvironment,
+    imported_fn::{
+        comments::{
+            add_leading_comment_proxy, add_leading_comments_proxy, add_pure_comment_proxy,
+            add_trailing_comment_proxy, add_trailing_comments_proxy, copy_comment_to_host_env,
+            get_leading_comments_proxy, get_trailing_comments_proxy, has_leading_comments_proxy,
+            has_trailing_comments_proxy, move_leading_comments_proxy, move_trailing_comments_proxy,
+            take_leading_comments_proxy, take_trailing_comments_proxy, CommentHostEnvironment,
+        },
+        set_transform_result::{set_transform_result, TransformResultHostEnvironment},
+    },
+};
 
 mod comments;
 mod handler;
@@ -55,7 +67,6 @@ mod set_transform_result;
 
 use handler::*;
 use hygiene::*;
-use set_transform_result::*;
 
 /// Create an ImportObject includes functions to be imported from host to the
 /// plugins.
@@ -68,16 +79,13 @@ pub(crate) fn build_import_object(
     // transfrom_result
     let set_transform_result_fn_decl = Function::new_native_with_env(
         wasmer_store,
-        HostEnvironment::new(transform_result),
+        TransformResultHostEnvironment::new(transform_result),
         set_transform_result,
     );
 
     // handler
-    let emit_diagnostics_fn_decl = Function::new_native_with_env(
-        wasmer_store,
-        HostEnvironment::new(transform_result),
-        emit_diagnostics,
-    );
+    let emit_diagnostics_fn_decl =
+        Function::new_native_with_env(wasmer_store, BaseHostEnvironment::new(), emit_diagnostics);
 
     // hygiene
     let mark_fresh_fn_decl = Function::new_native(wasmer_store, mark_fresh_proxy);
@@ -86,13 +94,13 @@ pub(crate) fn build_import_object(
     let mark_set_builtin_fn_decl = Function::new_native(wasmer_store, mark_set_builtin_proxy);
     let mark_is_descendant_of_fn_decl = Function::new_native_with_env(
         wasmer_store,
-        HostEnvironment::new(transform_result),
+        BaseHostEnvironment::new(),
         mark_is_descendant_of_proxy,
     );
 
     let mark_least_ancestor_fn_decl = Function::new_native_with_env(
         wasmer_store,
-        HostEnvironment::new(transform_result),
+        BaseHostEnvironment::new(),
         mark_least_ancestor_proxy,
     );
 
@@ -100,19 +108,91 @@ pub(crate) fn build_import_object(
         Function::new_native(wasmer_store, syntax_context_apply_mark_proxy);
     let syntax_context_remove_mark_fn_decl = Function::new_native_with_env(
         wasmer_store,
-        HostEnvironment::new(transform_result),
+        BaseHostEnvironment::new(),
         syntax_context_remove_mark_proxy,
     );
     let syntax_context_outer_fn_decl =
         Function::new_native(wasmer_store, syntax_context_outer_proxy);
 
     // comments
+    let comment_buffer = Arc::new(Mutex::new(vec![]));
+
+    let copy_comment_to_host_env_fn_decl = Function::new_native_with_env(
+        wasmer_store,
+        CommentHostEnvironment::new(&comment_buffer),
+        copy_comment_to_host_env,
+    );
+
+    let add_leading_comment_fn_decl = Function::new_native_with_env(
+        wasmer_store,
+        CommentHostEnvironment::new(&comment_buffer),
+        add_leading_comment_proxy,
+    );
+
+    let add_leading_comments_fn_decl = Function::new_native_with_env(
+        wasmer_store,
+        CommentHostEnvironment::new(&comment_buffer),
+        add_leading_comments_proxy,
+    );
+
+    let has_leading_comments_fn_decl =
+        Function::new_native(wasmer_store, has_leading_comments_proxy);
+
+    let move_leading_comments_fn_decl =
+        Function::new_native(wasmer_store, move_leading_comments_proxy);
+
+    let take_leading_comments_fn_decl = Function::new_native_with_env(
+        wasmer_store,
+        // take_* doesn't need to share buffer to pass values from plugin to the host - do not
+        // clone buffer here.
+        CommentHostEnvironment::new(&Default::default()),
+        take_leading_comments_proxy,
+    );
+
     let get_leading_comments_fn_decl = Function::new_native_with_env(
         wasmer_store,
-        HostEnvironment::new(transform_result),
+        // get_* doesn't need to share buffer to pass values from plugin to the host - do not clone
+        // buffer here.
+        CommentHostEnvironment::new(&Default::default()),
         get_leading_comments_proxy,
     );
 
+    let add_trailing_comment_fn_decl = Function::new_native_with_env(
+        wasmer_store,
+        CommentHostEnvironment::new(&comment_buffer),
+        add_trailing_comment_proxy,
+    );
+
+    let add_trailing_comments_fn_decl = Function::new_native_with_env(
+        wasmer_store,
+        CommentHostEnvironment::new(&comment_buffer),
+        add_trailing_comments_proxy,
+    );
+
+    let has_trailing_comments_fn_decl =
+        Function::new_native(wasmer_store, has_trailing_comments_proxy);
+
+    let move_trailing_comments_fn_decl =
+        Function::new_native(wasmer_store, move_trailing_comments_proxy);
+
+    let take_trailing_comments_fn_decl = Function::new_native_with_env(
+        wasmer_store,
+        // take_* doesn't need to share buffer to pass values from plugin to the host - do not
+        // clone buffer here.
+        CommentHostEnvironment::new(&Default::default()),
+        take_trailing_comments_proxy,
+    );
+
+    let get_trailing_comments_fn_decl = Function::new_native_with_env(
+        wasmer_store,
+        // get_* doesn't need to share buffer to pass values from plugin to the host - do not clone
+        // buffer here.
+        CommentHostEnvironment::new(&Default::default()),
+        get_trailing_comments_proxy,
+    );
+
+    let add_pure_comment_fn_decl = Function::new_native(wasmer_store, add_pure_comment_proxy);
+
     imports! {
         "env" => {
             // transform
@@ -130,7 +210,20 @@ pub(crate) fn build_import_object(
             "__syntax_context_remove_mark_proxy" => syntax_context_remove_mark_fn_decl,
             "__syntax_context_outer_proxy" => syntax_context_outer_fn_decl,
             // comments
+            "__copy_comment_to_host_env" => copy_comment_to_host_env_fn_decl,
+            "__add_leading_comment_proxy" => add_leading_comment_fn_decl,
+            "__add_leading_comments_proxy" => add_leading_comments_fn_decl,
+            "__has_leading_comments_proxy" => has_leading_comments_fn_decl,
+            "__move_leading_comments_proxy" => move_leading_comments_fn_decl,
+            "__take_leading_comments_proxy" => take_leading_comments_fn_decl,
             "__get_leading_comments_proxy" => get_leading_comments_fn_decl,
+            "__add_trailing_comment_proxy" => add_trailing_comment_fn_decl,
+            "__add_trailing_comments_proxy" => add_trailing_comments_fn_decl,
+            "__has_trailing_comments_proxy" => has_trailing_comments_fn_decl,
+            "__move_trailing_comments_proxy" => move_trailing_comments_fn_decl,
+            "__take_trailing_comments_proxy" => take_trailing_comments_fn_decl,
+            "__get_trailing_comments_proxy" => get_trailing_comments_fn_decl,
+            "__add_pure_comment_proxy" => add_pure_comment_fn_decl,
         }
     }
 }
